@@ -4,6 +4,7 @@ import android.text.TextUtils;
 import android.util.Log;
 
 import com.alibaba.fastjson.JSON;
+import com.alibaba.fastjson.JSONArray;
 import com.alibaba.fastjson.JSONObject;
 
 import java.io.BufferedReader;
@@ -1052,4 +1053,191 @@ public class Utils {
         }
         return Font;
     }
+
+    /*
+     * 更新存储当前收到的字幕信息。用于CosyVoice。
+     */
+    public static class ShowSubtitleInfo {
+        public int sentenceIndex;
+        public int maxEndIndex;
+        public String show_text;
+
+        public ShowSubtitleInfo() {
+            this.sentenceIndex = 0;
+            this.maxEndIndex = 0;
+            this.show_text = "";
+        }
+
+        public void clear() {
+            this.sentenceIndex = 0;
+            this.maxEndIndex = 0;
+            this.show_text = "";
+        }
+
+        public boolean setIndex(JSONObject obj, String arrayName) {
+            /*
+             * obj:
+             *  {"index":0,
+             *  "subtitles":[
+             *      {"begin_index":0,"begin_time":0,"end_index":1,"end_time":120,"phoneme":"null","phoneme_list":[],"sentence":false,"text":"唧"}
+             *  ]}
+             */
+            boolean changed = false;
+            if (obj == null) {
+                return changed;
+            }
+
+            this.sentenceIndex = obj.getInteger("index");
+            JSONArray words = obj != null ? obj.getJSONArray(arrayName) : null;
+            if (words != null && !words.isEmpty()) {
+                for (int i = 0; i < words.size(); i++) {
+                    JSONObject w = words.getJSONObject(i);
+                    if (w == null) continue;
+                    if (w.containsKey("end_index")) {
+                        int endIdx = w.getIntValue("end_index");
+                        String curChar = w.getString("text");
+                        if (endIdx > maxEndIndex) {
+                            maxEndIndex = endIdx;
+                            show_text += curChar;
+                            changed = true;
+                        }
+                    }
+                }
+            }
+            return changed;
+        }
+    }
+
+    /*
+     * 更新当前正在展示的字幕信息。用于CosyVoice。
+     */
+    public static class HighLightWordList {
+        public class HighLightWordNode {
+            public int begin_index;
+            public int begin_time;
+            public int end_index;
+            public int end_time;
+            public HighLightWordNode next;
+
+            public HighLightWordNode(int beginIdx, int endIdx, int beginTime, int endTime) {
+                this.begin_index = beginIdx;
+                this.begin_time = beginTime;
+                this.end_index = endIdx;
+                this.end_time = endTime;
+                this.next = null;
+            }
+        }
+
+        public int maxEndIndex;
+        public HighLightWordNode head;
+        public HighLightWordNode tail;
+        public int old_high_light_begin_index;
+        public int old_high_light_end_index;
+        public int high_light_begin_index;
+        public int high_light_end_index;
+
+        public HighLightWordList() {
+            this.maxEndIndex = 0;
+            this.head = null;
+            this.tail = null;
+            this.high_light_begin_index = 0;
+            this.high_light_end_index = 0;
+            this.old_high_light_begin_index = 0;
+            this.old_high_light_end_index = 0;
+        }
+
+        public void clear() {
+            this.maxEndIndex = 0;
+            this.head = null;
+            this.tail = null;
+            this.high_light_begin_index = 0;
+            this.high_light_end_index = 0;
+            this.old_high_light_begin_index = 0;
+            this.old_high_light_end_index = 0;
+        }
+
+        public boolean buildListFromJson(JSONObject obj, String arrayName) {
+            boolean changed = false;
+            if (obj == null) {
+                return changed;
+            }
+            JSONArray words = obj != null ? obj.getJSONArray(arrayName) : null;
+            if (words != null && !words.isEmpty()) {
+                for (int i = 0; i < words.size(); i++) {
+                    JSONObject w = words.getJSONObject(i);
+                    if (w.containsKey("end_index")) {
+                        int beginIdx = w.getIntValue("begin_index");
+                        int endIdx = w.getIntValue("end_index");
+                        int beginTime = w.getIntValue("begin_time");
+                        int endTime = w.getIntValue("end_time");
+                        if (endIdx > maxEndIndex) {
+                            changed = true;
+                            maxEndIndex = endIdx;
+                            HighLightWordNode node = new HighLightWordNode(beginIdx, endIdx, beginTime, endTime);
+                            if (head == null) {
+                                head = tail = node;
+                            } else {
+                                tail.next = node;
+                                tail = node;
+                            }
+                        }
+                    }
+                }
+            }
+            return changed;
+        }
+
+        public boolean flushHighLight(String json_str) {
+            boolean changed = false;
+            JSONObject obj = JSON.parseObject(json_str);
+            if (obj == null) {
+                return changed;
+            }
+            int playingBeginTime = obj.getIntValue("begin_time");
+            int playingEndTime = obj.getIntValue("end_time");
+            int cur_light_words_cnt = 0;
+            this.old_high_light_begin_index = this.high_light_begin_index;
+            this.old_high_light_end_index = this.high_light_end_index;
+            HighLightWordNode node = this.head;
+            /* 清除已经高亮过的节点 */
+            while (node != null) {
+                if (node.end_index < old_high_light_begin_index) {
+                    head = node.next;
+                }
+                node = node.next;
+            }
+            node = this.head;
+            while (node != null) {
+                if (cur_light_words_cnt == 0) {
+                    if (node.begin_time <= playingBeginTime && node.end_time >= playingBeginTime) {
+                        /* 当前音频首部落在某个字的时间戳上 */
+                        this.high_light_begin_index = node.begin_index;
+                        if (this.high_light_end_index == 0) {
+                            this.high_light_end_index = this.high_light_begin_index + 1;
+                        }
+                        cur_light_words_cnt++;
+                        changed = true;
+                    }
+                } else {
+                    if (node.begin_time <= playingEndTime && node.end_time >= playingEndTime) {
+                        /* 当前音频尾部落在某个字的时间戳上 */
+                        this.high_light_end_index = node.end_index;
+                        cur_light_words_cnt++;
+                        changed = true;
+                    } else {
+                        break;
+                    }
+                }
+                node = node.next;
+            } // while
+            if (changed) {
+                if (this.old_high_light_begin_index == this.high_light_begin_index &&
+                        this.old_high_light_end_index == this.high_light_end_index) {
+                    /* 如果和上次高亮的相同则这次跳过操作 */
+                    changed = false;
+                }
+            }
+            return changed;
+        }
+    }
 }
